import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.lang.String;

/**
 * An Ant may be pressed down, the mouse dragged and released to make the ant jump.
 * 
 * @author John Brink
 * @version March 2011
 */
public class Ant  extends SmoothMover

{
    public static final Vector GRAVITY = new Vector(90, 0.5);
    int initialX = -1;
    int initialY = -1;
    int finalX = -1;
    int finalY = -1;
    int heading = 270;
    boolean active = false;
    Vector headingVector = new Vector();
   
    String imageTag;
    String imageTagStretch;
    Target target;
    
    
    /**
     * Constructor for an ant, select a random color and place the ant
     */
    public Ant(Target target)
    {
        int color = Greenfoot.getRandomNumber(4) + 1;
        switch(color)
        {
            case 1: // Create a blue ant
            imageTag = "blueant.png";
            imageTagStretch = "blueantstretched.png";
            break;
            case 2: // Create a green ant
            imageTag = "greenant.png";
            imageTagStretch = "greenantstretched.png";
            break;
            case 3: // Create a red ant
            imageTag = "redant.png";
            imageTagStretch = "redantstretched.png";
            break;
            case 4: // Create a yellow ant
            imageTag = "yellowant.png";
            imageTagStretch = "yellowantstretched.png";
            break;
            default: // Create a blue ant as a default
            imageTag = "blueant.png";
            imageTagStretch = "blueantstretched.png";
            break;
        }
        setImage(imageTag);
        this.target = target;
        
    }
    
    /**
     * Act - do whatever the Ant wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        checkPress();
        checkDragEnded();
        if(active)
        {
            increaseSpeed(GRAVITY);
            move();
            tumble();
            checkForTarget();
            if (hitCeiling())
            {
                redirect(headingVector);
            }
            if (hitWall())
            {
                bounceOffWall();
            }
            if (hitFloor())
            {
                getWorld().removeObject(this);
            }

                 
        } 
       
    }
    
    public void checkPress()
    {
        
        
        if(Greenfoot.mousePressed(this))
        {

            // This will execute when the mouse button is pressed and dragged
            MouseInfo mouse;
            mouse = Greenfoot.getMouseInfo();
            
            // check to see if mouse press location has been registered yet
            
                initialX = mouse.getX();
                initialY = mouse.getY();
                setImage(imageTagStretch);
                
        }
            
                // Revert to normal image, if mouse clicked but not dragged
        if(Greenfoot.mouseClicked(this))
        {
            setImage(imageTag);
        }
    }
    
    public void checkDragEnded()
    {
        if(Greenfoot.mouseDragEnded(this))
        {
            // This will excute when the mouse button is released
            MouseInfo mouse;
            mouse = Greenfoot.getMouseInfo();
        
            // get the final location of mouse when button released
            finalX = mouse.getX();
            finalY = mouse.getY();
            setImage(imageTag);
            // get the direction and the speed (length of dragged vector)
            active = true;
            headingVector.setDirection(getDirection());
            headingVector.setLength(getPower());
            increaseSpeed(headingVector);
            target.antCount++;
            
        } 
        
    }
    
  
    /**
     * Based on the direction the mouse was dragged from the ant, determine a heading direction 
     * for the ant to jump
     */
    public int getDirection()
    {
        int deltaX = finalX - initialX;
        int deltaY = finalY - initialY;
        int newDirection;
        
        newDirection = (int) Math.toDegrees(Math.atan2(deltaY, deltaX));
        // Add 180 degrees because direction of ant is opposite direction of drag
        // newDirection = (newDirection + 180) % 360;
        return newDirection;
    }
    
    /**
     * Based on the length of the mouse drag from the ant, determine an initial speed at which the ant
     * will jump
     */
    public double getPower()
    {
        int deltaX = finalX - initialX;
        int deltaY = finalY - initialY;
        double power;
        
        // Use the pythagorean theorem to determine length of hypotenuse (length of drag).
        // in opposite directon
        power = -(Math.sqrt((deltaX*deltaX) + (deltaY*deltaY)));
        // Scale the raw length of drag for desired speed effect
        power *= 1.5;
        return power/10;
    }

    public void tumble()
    {
        setRotation(getRotation() - 20);
    }
    
    public boolean hitCeiling()
    {
        return (this.getY() <= 20);
    }
    
    public void redirect(Vector vector)
    {
        int newHeading;
        double newSpeed;
        Vector redirectVector = vector.copy();
        
        newHeading = (90 -((vector.getDirection() + 180) % 360));
        newSpeed = redirectVector.getLength()*.3;
        redirectVector.setDirection(newHeading);
        redirectVector.setLength(newSpeed);
        
        this.setSpeed(redirectVector);
        move();
        
        

    }
    
    public boolean hitFloor()
    {
        return (this.getY() >= getWorld().getHeight() - 25);
    }
    
    public boolean hitWall()
    {
        return (this.getX() < 40 || this.getX() > getWorld().getWidth()-40);
    }
    
    public void bounceOffWall()
    {
        Vector deadWeight = headingVector.copy();
        int newDirection = 270; //(deadWeight.getDirection() + 225) % 360;
        deadWeight.setDirection(newDirection);
        deadWeight.setLength(deadWeight.getLength()*.5);
                
        this.setSpeed(deadWeight);
        

        
    }
    /**
     *   This is only for testing
     */
    public void checkForTarget()
    {
        Vector mySpeed = this.getSpeed();
        
        if(getOneIntersectingObject(Target.class) != null)
        {
            if (mySpeed.getY() <= 0)
            {
                target.setGoingDown(false);
            } else
            {
                target.setGoingDown(true);
            }
        }
    }   

    
}
